/******************************************************************************
 * Copyright (c) 1998 by Mindbright Technology AB, Stockholm, Sweden.
 *
 *                        All Rights Reserved
 *
 *               www.mindbright.se, info@mindbright.se
 *
 * This program and its source-code may be used FREE of charge for strictly
 * NON-COMMERCIAL purposes. Companies or persons wishing to make non-commercial
 * software that in whole or in part contains or is derived from this program
 * or any part thereof must inform the original author of such work. The
 * original author must also be given attribution as the author of used parts.
 * For COMMERCIAL use all parts of this program originally developed by
 * Mindbright Technology must be LICENSED under Mindbright Technology's
 * SHAREWARE license, please contact Mindbright Technology for information on
 * this. This copyright notice must allways be included and unchanged.
 *
 * IF LICENSED FREE OF CHARGE AND/OR IF NOT STATED OTHERWISE THIS PROGRAM COMES
 * WITH NO WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, IN NO EVENT SHALL
 * MINDBRIGHT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *****************************************************************************/
package mindbright.security;

import java.math.BigInteger;
import java.security.*;

public class RSACipher {

  public KeyPair keys;

  public RSACipher(KeyPair keys) {
    this.keys = keys;
  }

  public BigInteger doPublic(BigInteger input) {
    RSAPublicKey pubKey = (RSAPublicKey)keys.getPublic();
    BigInteger   result;

    result = input.modPow(pubKey.getE(), pubKey.getN());

    return result;
  }

  public BigInteger doPrivate(BigInteger input) {
    BigInteger    dp;
    BigInteger    dq;
    BigInteger    p2;
    BigInteger    q2;
    BigInteger    k;
    BigInteger    result;
    BigInteger    one = new BigInteger("1");
    RSAPrivateKey privKey = (RSAPrivateKey)keys.getPrivate();

    dp = privKey.getP().subtract(one);
    dp = privKey.getD().mod(dp);

    dq = privKey.getQ().subtract(one);
    dq = privKey.getD().mod(dq);

    p2 = input.mod(privKey.getP());
    p2 = p2.modPow(dp, privKey.getP());

    q2 = input.mod(privKey.getQ());
    q2 = q2.modPow(dq, privKey.getQ());

    if(p2.compareTo(q2) == 0)
      return p2;

    k = q2.subtract(p2).mod(privKey.getQ());
    k = k.multiply(privKey.getU());
    k = k.mod(privKey.getQ());

    result = k.multiply(privKey.getP());
    result = result.add(p2);

    return result;
  }

  public static void main(String[] argv) {
    KeyPair    kp;
    RSACipher  cipher;
    BigInteger p;
    BigInteger q;
    BigInteger t;
    BigInteger p_1;
    BigInteger q_1;
    BigInteger phi;
    BigInteger G;
    BigInteger F;
    BigInteger e;
    BigInteger d;
    BigInteger u;
    BigInteger n;
    BigInteger one = new BigInteger("1");

    System.out.println("Generating primes...");

    //    p = new BigInteger(128, 64, new SecureRandom());
    p = new BigInteger("3336670033");
    //    q = new BigInteger(128, 64, new SecureRandom());
    q = new BigInteger("9876543211");

    if(p.compareTo(q) == 0) {
      System.out.println("Same prime, impossible!!!");
      System.exit(0);
    } else if(q.compareTo(p) < 0) {
      t = q;
      q = p;
      p = t;
    }

    t = p.gcd(q);
    if(t.compareTo(one) != 0) {
      System.out.println("Same prime, impossible!!!");
      System.exit(0);
    }

    p_1 = p.subtract(one);
    q_1 = q.subtract(one);
    phi = p_1.multiply(q_1);
    G   = p_1.gcd(q_1);
    F   = phi.divide(G);

    e   = one.shiftLeft(5);
    e   = e.subtract(one);
    do {
      e = e.add(one.add(one));
      t = e.gcd(phi);
    } while(t.compareTo(one) != 0);

    //    d = e.modInverse(F);
    d = e.modInverse(phi);
    n = p.multiply(q);
    u = p.modInverse(q);

    kp = new KeyPair(new RSAPublicKey(e, n),
		     new RSAPrivateKey(e, n, d, u, p, q));
    cipher = new RSACipher(kp);

    System.out.println("..p = " + p.toString());
    System.out.println("..q = " + q.toString());
    System.out.println("..phi = " + phi.toString());
    System.out.println("..e = " + e.toString());
    System.out.println("..n = " + n.toString());
    System.out.println("..d = " + d.toString());
    System.out.println("..u = " + u.toString());


    //    one = new BigInteger(256, new SecureRandom());
    one = new BigInteger("1901211401001920");

    System.out.println("..val = " + one.toString());
    one = cipher.doPublic(one);
    System.out.println("..val = " + one.toString());
    one = cipher.doPrivate(one);
    System.out.println("..val = " + one.toString());

  }

}
