/******************************************************************************
 * Copyright (c) 1998 by Mindbright Technology AB, Stockholm, Sweden.
 *
 *                        All Rights Reserved
 *
 *               www.mindbright.se, info@mindbright.se
 *
 * This program and its source-code may be used FREE of charge for strictly
 * NON-COMMERCIAL purposes. Companies or persons wishing to make non-commercial
 * software that in whole or in part contains or is derived from this program
 * or any part thereof must inform the original author of such work. The
 * original author must also be given attribution as the author of used parts.
 * For COMMERCIAL use all parts of this program originally developed by
 * Mindbright Technology must be LICENSED under Mindbright Technology's
 * SHAREWARE license, please contact Mindbright Technology for information on
 * this. This copyright notice must allways be included and unchanged.
 *
 * IF LICENSED FREE OF CHARGE AND/OR IF NOT STATED OTHERWISE THIS PROGRAM COMES
 * WITH NO WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, IN NO EVENT SHALL
 * MINDBRIGHT BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
 * IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *****************************************************************************/
package mindbright.ssh;

import java.net.*;
import java.io.*;

public class SSHListenChannel extends SSHChannel {
  SSHChannelController controller;

  ServerSocket listenSocket;
  String       remoteHost;
  int          remotePort;

  boolean temporaryListener;

  public SSHListenChannel(int localPort, String remoteHost, int remotePort,
			  SSHChannelController controller)
    throws IOException {
    super(SSH.LISTEN_CHAN_NUM);
    this.controller    = controller;
    this.listenSocket  = new ServerSocket(localPort);
    this.remoteHost    = remoteHost;
    this.remotePort    = remotePort;
  }

  public int getListenPort() {
    return listenSocket.getLocalPort();
  }

  public SSHTunnel newTunnel(Socket ioSocket, int channelId, int remoteChannelId,
			     SSHChannelController controller) throws IOException {
    return new SSHTunnel(ioSocket, channelId, remoteChannelId, controller);
  }

  public void setTemporaryListener(boolean val) {
    temporaryListener = val;
  }

  public void serviceLoop() throws IOException {

    System.out.println("Starting listen-chan: " + listenSocket.getLocalPort());

    for(;;) {
      Socket fwdSocket = listenSocket.accept();
      SSHPduOutputStream respPdu = new SSHPduOutputStream(SSH.MSG_PORT_OPEN, controller.sndCipher);

      int newChan      = controller.newChannelId();
      SSHTunnel tunnel = newTunnel(fwdSocket,
				   newChan, SSH.UNKNOWN_CHAN_NUM,
				   controller);
      controller.addTunnel(tunnel);

      respPdu.writeInt(newChan);
      respPdu.writeString(remoteHost);
      respPdu.writeInt(remotePort);

      System.out.println("got connect for: " + remoteHost + " : " + remotePort + ", " + newChan);

      //
      // !!! TODO: check this !!! if(controller.haveHostInFwdOpen())
      //
      respPdu.writeString(fwdSocket.getInetAddress().getHostAddress());

      controller.transmit(respPdu);

      if(temporaryListener)
	break;

    }
  }

}
